@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2005 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2010 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,8 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
 """HyperArch: Pipermail archiving for Mailman
 
@@ -45,6 +46,7 @@ from email.Charset import Charset
 
 from Mailman import mm_cfg
 from Mailman import Utils
+from Mailman import Errors
 from Mailman import LockFile
 from Mailman import MailList
 from Mailman import i18n
@@ -154,7 +156,12 @@ REpat = re.compile( r"\s*RE\s*(\[\d+\]\s*)?:\s*", re.IGNORECASE)
 emailpat = re.compile(r'([-+,.\w]+@[-+.\w]+)')
 
 #  Argh!  This pattern is buggy, and will choke on URLs with GET parameters.
-urlpat = re.compile(r'(\w+://[^>)\s]+)') # URLs in text
+# MAS: Given that people are not constrained in how they write URIs in plain
+# text, it is not possible to have a single regexp to reliably match them.
+# The regexp below is intended to match straightforward cases.  Even humans
+# can't reliably tell whether various punctuation at the end of a URI is part
+# of the URI or not.
+urlpat = re.compile(r'([a-z]+://.*?)(?:_\s|_$|$|[]})>\'"\s])', re.IGNORECASE)
 
 # Blank lines
 blankpat = re.compile(r'^\s*$')
@@ -278,7 +285,11 @@ class Article(pipermail.Article):
             otrans = i18n.get_translation()
             try:
                 i18n.set_language(lang)
-                self.email = re.sub('@', _(' at '), self.email)
+                if self.author == self.email:
+                    self.author = self.email = re.sub('@', _(' at '),
+                                                      self.email)
+                else:
+                    self.email = re.sub('@', _(' at '), self.email)
             finally:
                 i18n.set_translation(otrans)
 
@@ -291,6 +302,9 @@ class Article(pipermail.Article):
         self.decoded = {}
         cset = Utils.GetCharSet(mlist.preferred_language)
         cset_out = Charset(cset).output_charset or cset
+        if isinstance(cset_out, unicode):
+            # email 3.0.1 (python 2.4) doesn't like unicode
+            cset_out = cset_out.encode('us-ascii')
         charset = message.get_content_charset(cset_out)
         if charset:
             charset = charset.lower().strip()
@@ -402,6 +416,15 @@ class Article(pipermail.Article):
             if email:
                 self.decoded['email'] = email
         if subject:
+            if mm_cfg.ARCHIVER_OBSCURES_EMAILADDRS:
+                otrans = i18n.get_translation()
+                try:
+                    i18n.set_language(self._lang)
+                    atmark = unicode(_(' at '), Utils.GetCharSet(self._lang))
+                    subject = re.sub(r'([-+,.\w]+)@([-+.\w]+)',
+                              '\g<1>' + atmark + '\g<2>', subject)
+                finally:
+                    i18n.set_translation(otrans)
             self.decoded['subject'] = subject
         self.decoded['stripped'] = self.strip_subject(subject or self.subject)
 
@@ -415,8 +438,11 @@ class Article(pipermail.Article):
             prefix_pat = re.sub(r'%\d*d', r'\s*\d+\s*', prefix_pat)
             subject = re.sub(prefix_pat, '', subject)
         subject = subject.lstrip()
-        strip_pat = re.compile('^((RE|AW|SV)(\[\d+\])?:\s*)+', re.I)
+        # MAS Should we strip FW and FWD too?
+        strip_pat = re.compile('^((RE|AW|SV|VS)(\[\d+\])?:\s*)+', re.I)
         stripped = strip_pat.sub('', subject)
+        # Also remove whitespace to avoid folding/unfolding differences
+        stripped = re.sub('\s', '', stripped)
         return stripped
 
     def decode_charset(self, field):
@@ -445,8 +471,10 @@ class Article(pipermail.Article):
             d["email_html"] = self.quote(self.email)
             d["title"] = self.quote(self.subject)
             d["subject_html"] = self.quote(self.subject)
-            d["subject_url"] = url_quote(self.subject)
-            d["in_reply_to_url"] = url_quote(self.in_reply_to)
+            # TK: These two _url variables are used to compose a response
+            # from the archive web page.  So, ...
+            d["subject_url"] = url_quote('Re: ' + self.subject)
+            d["in_reply_to_url"] = url_quote(self._message_id)
             if mm_cfg.ARCHIVER_OBSCURES_EMAILADDRS:
                 # Point the mailto url back to the list
                 author = re.sub('@', _(' at '), self.author)
@@ -550,16 +578,21 @@ class Article(pipermail.Article):
         if d['_message_id']:
             headers.append('Message-ID: %(_message_id)s')
         body = EMPTYSTRING.join(self.body)
-        if isinstance(body, types.UnicodeType):
-            body = body.encode(Utils.GetCharSet(self._lang), 'replace')
+        cset = Utils.GetCharSet(self._lang)
+        # Coerce the body to Unicode and replace any invalid characters.
+        if not isinstance(body, types.UnicodeType):
+            body = unicode(body, cset, 'replace')
         if mm_cfg.ARCHIVER_OBSCURES_EMAILADDRS:
             otrans = i18n.get_translation()
             try:
                 i18n.set_language(self._lang)
+                atmark = unicode(_(' at '), cset)
                 body = re.sub(r'([-+,.\w]+)@([-+.\w]+)',
-                              '\g<1>' + _(' at ') + '\g<2>', body)
+                              '\g<1>' + atmark + '\g<2>', body)
             finally:
                 i18n.set_translation(otrans)
+        # Return body to character set of article.
+        body = body.encode(cset, 'replace')
         return NL.join(headers) % d + '\n\n' + body + '\n'
 
     def _set_date(self, message):
@@ -666,6 +699,7 @@ class HyperArchive(pipermail.T):
                  "archivedate": quotetime(self.archivedate),
                  "listinfo": mlist.GetScriptURL('listinfo', absolute=1),
                  "version": self.version,
+                 "listname": html_quote(mlist.real_name, self.lang),
                  }
             i = {"thread": _("thread"),
                  "subject": _("subject"),
