@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2006 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2010 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,9 +27,9 @@ the mailing lists, and whatever else doesn't belong elsewhere.
 from __future__ import nested_scopes
 
 import os
+import sys
 import re
 import cgi
-import sha
 import time
 import errno
 import base64
@@ -55,6 +55,16 @@ from Mailman import Site
 from Mailman.SafeDict import SafeDict
 from Mailman.Logging.Syslog import syslog
 
+try:
+    import hashlib
+    md5_new = hashlib.md5
+    sha_new = hashlib.sha1
+except ImportError:
+    import md5
+    import sha
+    md5_new = md5.new
+    sha_new = sha.new
+
 try:
     True, False
 except NameError:
@@ -203,6 +213,9 @@ def LCDomain(addr):
 
 # TBD: what other characters should be disallowed?
 _badchars = re.compile(r'[][()<>|;^,\000-\037\177-\377]')
+# characters in addition to _badchars which are not allowed in
+# unquoted local parts.
+_specials = re.compile(r'[:\\"]')
 
 def ValidateEmail(s):
     """Verify that an email address isn't grossly evil."""
@@ -212,11 +225,15 @@ def ValidateEmail(s):
     if _badchars.search(s) or s[0] == '-':
         raise Errors.MMHostileAddress, s
     user, domain_parts = ParseEmail(s)
-    # This means local, unqualified addresses, are no allowed
+    # This means local, unqualified addresses, are not allowed
     if not domain_parts:
         raise Errors.MMBadEmailError, s
     if len(domain_parts) < 2:
         raise Errors.MMBadEmailError, s
+    if not (user.startswith('"') and user.endswith('"')):
+        # local part is not quoted so it can't contain specials
+        if _specials.search(user):
+            raise Errors.MMBadEmailError, s
 
 
 
@@ -249,7 +266,7 @@ def ScriptURL(target, web_page_url=None, absolute=False):
         fullpath = os.environ.get('SCRIPT_NAME', '') + \
                    os.environ.get('PATH_INFO', '')
     baseurl = urlparse.urlparse(web_page_url)[2]
-    if not absolute and fullpath.endswith(baseurl):
+    if not absolute and fullpath.startswith(baseurl):
         # Use relative addressing
         fullpath = fullpath[len(baseurl):]
         i = fullpath.find('?')
@@ -377,7 +394,7 @@ def set_global_password(pw, siteadmin=True):
     omask = os.umask(026)
     try:
         fp = open(filename, 'w')
-        fp.write(sha.new(pw).hexdigest() + '\n')
+        fp.write(sha_new(pw).hexdigest() + '\n')
         fp.close()
     finally:
         os.umask(omask)
@@ -403,12 +420,14 @@ def check_global_password(response, siteadmin=True):
     challenge = get_global_password(siteadmin)
     if challenge is None:
         return None
-    return challenge == sha.new(response).hexdigest()
+    return challenge == sha_new(response).hexdigest()
 
 
 
+_ampre = re.compile('&amp;((?:#[0-9]+|[a-z]+);)', re.IGNORECASE)
 def websafe(s):
-    return cgi.escape(s, quote=True)
+    # Don't double escape html entities
+    return _ampre.sub(r'&\1', cgi.escape(s, quote=True))
 
 
 def nntpsplit(s):
@@ -574,7 +593,7 @@ ADMINDATA = {
     'set':         (3, 3),
     'subscribe':   (0, 3),
     'unsubscribe': (0, 1),
-    'who':         (0, 0),
+    'who':         (0, 2),
     }
 
 # Given a Message.Message object, test for administrivia (eg subscribe,
@@ -674,6 +693,9 @@ def GetLanguageDescr(lang):
 def GetCharSet(lang):
     return mm_cfg.LC_DESCRIPTIONS[lang][1]
 
+def GetDirection(lang):
+    return mm_cfg.LC_DESCRIPTIONS[lang][2]
+
 def IsLanguage(lang):
     return mm_cfg.LC_DESCRIPTIONS.has_key(lang)
 
@@ -788,12 +810,25 @@ def canonstr(s, lang=None):
     newparts = []
     parts = re.split(r'&(?P<ref>[^;]+);', s)
     def appchr(i):
-        if i < 256:
-            newparts.append(chr(i))
+        # do everything in unicode
+        newparts.append(unichr(i))
+    def tounicode(s):
+        # We want the default fallback to be iso-8859-1 even if the language
+        # is English (us-ascii).  This seems like a practical compromise so
+        # that non-ASCII characters in names can be used in English lists w/o
+        # having to change the global charset for English from us-ascii (which
+        # I superstitiously think may have unintended consequences).
+        if isinstance(s, unicode):
+            return s
+        if lang is None:
+            charset = 'iso-8859-1'
         else:
-            newparts.append(unichr(i))
+            charset = GetCharSet(lang)
+            if charset == 'us-ascii':
+                charset = 'iso-8859-1'
+        return unicode(s, charset, 'replace')
     while True:
-        newparts.append(parts.pop(0))
+        newparts.append(tounicode(parts.pop(0)))
         if not parts:
             break
         ref = parts.pop(0)
@@ -802,28 +837,16 @@ def canonstr(s, lang=None):
                 appchr(int(ref[1:]))
             except ValueError:
                 # Non-convertable, stick with what we got
-                newparts.append('&'+ref+';')
+                newparts.append(tounicode('&'+ref+';'))
         else:
             c = htmlentitydefs.entitydefs.get(ref, '?')
             if c.startswith('#') and c.endswith(';'):
                 appchr(int(ref[1:-1]))
             else:
-                newparts.append(c)
+                newparts.append(tounicode(c))
     newstr = EMPTYSTRING.join(newparts)
-    if isinstance(newstr, UnicodeType):
-        return newstr
-    # We want the default fallback to be iso-8859-1 even if the language is
-    # English (us-ascii).  This seems like a practical compromise so that
-    # non-ASCII characters in names can be used in English lists w/o having to
-    # change the global charset for English from us-ascii (which I
-    # superstitiously think may have unintended consequences).
-    if lang is None:
-        charset = 'iso-8859-1'
-    else:
-        charset = GetCharSet(lang)
-        if charset == 'us-ascii':
-            charset = 'iso-8859-1'
-    return unicode(newstr, charset, 'replace')
+    # newstr is unicode
+    return newstr
 
 
 # The opposite of canonstr() -- sorta.  I.e. it attempts to encode s in the
@@ -873,3 +896,157 @@ def oneline(s, cset):
     except (LookupError, UnicodeError, ValueError, HeaderParseError):
         # possibly charset problem. return with undecoded string in one line.
         return EMPTYSTRING.join(s.splitlines())
+
+
+# Patterns and functions to flag possible XSS attacks in HTML.
+# This list is compiled from information at http://ha.ckers.org/xss.html,
+# http://www.quirksmode.org/js/events_compinfo.html,
+# http://www.htmlref.com/reference/appa/events1.htm,
+# http://lxr.mozilla.org/mozilla/source/content/events/src/nsDOMEvent.cpp#59,
+# http://www.w3.org/TR/DOM-Level-2-Events/events.html and
+# http://www.xulplanet.com/references/elemref/ref_EventHandlers.html
+# Many thanks are due to Moritz Naumann for his assistance with this.
+_badwords = [
+    '<i?frame',
+    # Kludge to allow the specific tag that's in the options.html template.
+    '<link(?! rel="SHORTCUT ICON" href="<mm-favicon>">)',
+    '<meta',
+    '<object',
+    '<script',
+    r'\bj(?:ava)?script\b',
+    r'\bvbs(?:cript)?\b',
+    r'\bdomactivate\b',
+    r'\bdomattrmodified\b',
+    r'\bdomcharacterdatamodified\b',
+    r'\bdomfocus(?:in|out)\b',
+    r'\bdommenuitem(?:in)?active\b',
+    r'\bdommousescroll\b',
+    r'\bdomnodeinserted(?:intodocument)?\b',
+    r'\bdomnoderemoved(?:fromdocument)?\b',
+    r'\bdomsubtreemodified\b',
+    r'\bfscommand\b',
+    r'\bonabort\b',
+    r'\bon(?:de)?activate\b',
+    r'\bon(?:after|before)print\b',
+    r'\bon(?:after|before)update\b',
+    r'\bonbefore(?:(?:de)?activate|copy|cut|editfocus|paste)\b',
+    r'\bonbeforeunload\b',
+    r'\bonbegin\b',
+    r'\bonblur\b',
+    r'\bonbounce\b',
+    r'\bonbroadcast\b',
+    r'\bon(?:cell)?change\b',
+    r'\boncheckboxstatechange\b',
+    r'\bon(?:dbl)?click\b',
+    r'\bonclose\b',
+    r'\boncommand(?:update)?\b',
+    r'\boncomposition(?:end|start)\b',
+    r'\boncontextmenu\b',
+    r'\boncontrolselect\b',
+    r'\boncopy\b',
+    r'\boncut\b',
+    r'\bondataavailable\b',
+    r'\bondataset(?:changed|complete)\b',
+    r'\bondrag(?:drop|end|enter|exit|gesture|leave|over)?\b',
+    r'\bondragstart\b',
+    r'\bondrop\b',
+    r'\bonend\b',
+    r'\bonerror(?:update)?\b',
+    r'\bonfilterchange\b',
+    r'\bonfinish\b',
+    r'\bonfocus(?:in|out)?\b',
+    r'\bonhelp\b',
+    r'\boninput\b',
+    r'\bonkey(?:up|down|press)\b',
+    r'\bonlayoutcomplete\b',
+    r'\bon(?:un)?load\b',
+    r'\bonlosecapture\b',
+    r'\bonmedia(?:complete|error)\b',
+    r'\bonmouse(?:down|enter|leave|move|out|over|up|wheel)\b',
+    r'\bonmove(?:end|start)?\b',
+    r'\bon(?:off|on)line\b',
+    r'\bonoutofsync\b',
+    r'\bonoverflow(?:changed)?\b',
+    r'\bonpage(?:hide|show)\b',
+    r'\bonpaint\b',
+    r'\bonpaste\b',
+    r'\bonpause\b',
+    r'\bonpopup(?:hidden|hiding|showing|shown)\b',
+    r'\bonprogress\b',
+    r'\bonpropertychange\b',
+    r'\bonradiostatechange\b',
+    r'\bonreadystatechange\b',
+    r'\bonrepeat\b',
+    r'\bonreset\b',
+    r'\bonresize(?:end|start)?\b',
+    r'\bonresume\b',
+    r'\bonreverse\b',
+    r'\bonrow(?:delete|enter|exit|inserted)\b',
+    r'\bonrows(?:delete|enter|inserted)\b',
+    r'\bonscroll\b',
+    r'\bonseek\b',
+    r'\bonselect(?:start)?\b',
+    r'\bonselectionchange\b',
+    r'\bonstart\b',
+    r'\bonstop\b',
+    r'\bonsubmit\b',
+    r'\bonsync(?:from|to)preference\b',
+    r'\bonsyncrestored\b',
+    r'\bontext\b',
+    r'\bontimeerror\b',
+    r'\bontrackchange\b',
+    r'\bonunderflow\b',
+    r'\bonurlflip\b',
+    r'\bseeksegmenttime\b',
+    r'\bsvgabort\b',
+    r'\bsvgerror\b',
+    r'\bsvgload\b',
+    r'\bsvgresize\b',
+    r'\bsvgscroll\b',
+    r'\bsvgunload\b',
+    r'\bsvgzoom\b',
+    ]
+
+
+# This is the actual re to look for the above patterns
+_badhtml = re.compile('|'.join(_badwords), re.IGNORECASE)
+# This is used to filter non-printable us-ascii characters, some of which
+# can be used to break words to avoid recognition.
+_filterchars = re.compile('[\000-\011\013\014\016-\037\177-\237]')
+# This is used to recognize '&#' and '%xx' strings for _translate which
+# translates them to characters
+_encodedchars = re.compile('(&#[0-9]+;?)|(&#x[0-9a-f]+;?)|(%[0-9a-f]{2})',
+                           re.IGNORECASE)
+
+
+def _translate(mo):
+    """Translate &#... and %xx encodings into the encoded character."""
+    match = mo.group().lower().strip('&#;')
+    try:
+        if match.startswith('x') or match.startswith('%'):
+            val = int(match[1:], 16)
+        else:
+            val = int(match, 10)
+    except ValueError:
+        return ''
+    if val < 256:
+        return chr(val)
+    else:
+        return ''
+
+
+def suspiciousHTML(html):
+    """Check HTML string for various tags, script language names and
+    'onxxx' actions that can be used in XSS attacks.
+    Currently, this a very simple minded test.  It just looks for
+    patterns without analyzing context.  Thus, it potentially flags lots
+    of benign stuff.
+    Returns True if anything suspicious found, False otherwise.
+    """
+
+    if _badhtml.search(_filterchars.sub(
+                       '', _encodedchars.sub(_translate, html))):
+        return True
+    else:
+        return False
+
